@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 
 # Python module: Client ModBus / TCP class 1
-#       Version: 0.0.5
+#       Version: 0.0.6
 #       Website: https://github.com/sourceperl/pyModbusTCP
-#          Date: 2014-08-04
+#          Date: 2014-08-25
 #       License: MIT (http://http://opensource.org/licenses/mit-license.php)
 #   Description: Client ModBus / TCP
 #                Support functions 3 and 16 (class 0)
 #                1,2,4,5,6 (Class 1)
 #       Charset: utf-8
 
-from pyModbusTCP import const
+from pyModbusTCP import constants as const
 import re
 import socket
 import select
@@ -18,16 +18,27 @@ import struct
 import random
 
 class ModbusClient:
-    """
-    Client Modbus TCP
-    """
+    """Client Modbus TCP"""
 
-    def __init__(self):
+    def __init__(self, host=None, port=None, debug=None):
         """Constructor
 
+        Modbus server params (host, port) can be set here or with host(), port()
+        functions. Same for debug option. 
+
+        Use functions avoid to launch ValueError except if params is incorrect.
+
+        :param host: hostname or IPv4/IPv6 address server address (optional)
+        :type host: str
+        :param port: TCP port number (optional)
+        :type port: int
+        :param debug: debug state (optional)
+        :type debug: bool
         :return: Object ModbusClient
         :rtype: ModbusClient
+        :raises ValueError: host, port or debug value is set but incorrect
         """
+        # object vars
         self.__hostname    = "localhost"
         self.__port        = const.MODBUS_PORT
         self.__unit_id     = 1
@@ -39,6 +50,18 @@ class ModbusClient:
         self.__version     = const.VERSION     # version number
         self.__last_error  = const.MB_NO_ERR   # last error code
         self.__last_except = 0                 # last expect code
+        # set host
+        if host:
+            if not self.host(host):
+                raise ValueError("host value error")
+        # set port
+        if port:
+            if not self.port(port):
+                raise ValueError("port value error")
+        # set hostname
+        if debug:
+            if not self.debug(debug):
+                raise ValueError("debug value error")
 
     def version(self):
         """Get package version
@@ -159,10 +182,9 @@ class ModbusClient:
     def open(self):
         """Connect to modbus server (open TCP connection)
 
-        :returns: True if connect or None if error
-        :rtype: bool or None if error
+        :returns: connect status (True if open)
+        :rtype: bool
         """
-        self.__debug_msg("call open()")
         # restart TCP if already open
         if self.is_open():
             self.close()
@@ -178,19 +200,21 @@ class ModbusClient:
                 self.__sock = socket.socket(af, socktype, proto)
             except socket.error:
                 self.__sock = None
-                self.__last_error = const.MB_CONNECT_ERR
-                self.__debug_msg("init socket error")
                 continue
             try:
                 self.__sock.connect(sa)
             except socket.error:
                 self.__sock.close()
                 self.__sock = None
-                self.__last_error = const.MB_CONNECT_ERR
-                self.__debug_msg("socket error")
                 continue
             break
-        return self.__sock is not None
+        # check connect status
+        if self.__sock is not None:
+            return True
+        else:
+            self.__last_error = const.MB_CONNECT_ERR
+            self.__debug_msg("connect error")
+            return False
 
     def is_open(self):
         """Get status of TCP connection
@@ -218,7 +242,7 @@ class ModbusClient:
 
         :param bit_addr: bit address (0 to 65535)
         :type bit_addr: int
-        :param bit_nb: number of bits to read (1 to 125)
+        :param bit_nb: number of bits to read (1 to 2000)
         :type bit_nb: int
         :returns: bits list or None if error
         :rtype: list of bool or None
@@ -227,7 +251,7 @@ class ModbusClient:
         if not (0 <= int(bit_addr) <= 65535):
             self.__debug_msg("read_coils() : bit_addr out of range")
             return None
-        if not (1 <= int(bit_nb) <= 125):
+        if not (1 <= int(bit_nb) <= 2000):
             self.__debug_msg("read_coils() : bit_nb out of range")
             return None
         if (int(bit_addr) + int(bit_nb)) > 65536:
@@ -252,23 +276,31 @@ class ModbusClient:
             self.__debug_msg("read_coils(): rx frame under min size")
             self.close()
             return None
-        # register extract
-        rx_byte_count = struct.unpack("B", f_body[0:1])
+        # extract field "byte count"
+        rx_byte_count = struct.unpack("B", f_body[0:1])[0]
         # frame with bits value -> bits[] list
-        f_bits = f_body[1:]
-        bits = []
-        for f_byte in bytearray(f_bits):
-            for pos in range(8):
-                bits.append(bool(f_byte>>pos&0x01))
-        # return only bit_nb bits
-        return bits[:int(bit_nb)]
+        f_bits = bytearray(f_body[1:])
+        # check rx_byte_count: match nb of bits request and check buffer size
+        if not ((rx_byte_count == int((bit_nb+7)/8)) or
+                (rx_byte_count == len(f_bits))):
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("read_coils(): rx byte count mismatch")
+            self.close()
+            return None
+        # allocate a bit_nb size list
+        bits = [None] * bit_nb
+        # fill bits list with bit items
+        for i, item in enumerate(bits):
+            bits[i] = bool(f_bits[int(i/8)]>>(i%8)&0x01)
+        # return bits list
+        return bits
 
     def read_discrete_inputs(self, bit_addr, bit_nb=1):
         """Modbus function READ_DISCRETE_INPUTS (0x02)
 
         :param bit_addr: bit address (0 to 65535)
         :type bit_addr: int
-        :param bit_nb: number of bits to read (1 to 125)
+        :param bit_nb: number of bits to read (1 to 2000)
         :type bit_nb: int
         :returns: bits list or None if error
         :rtype: list of bool or None
@@ -277,7 +309,7 @@ class ModbusClient:
         if not (0 <= int(bit_addr) <= 65535):
             self.__debug_msg("read_discrete_inputs() : bit_addr out of range")
             return None
-        if not (1 <= int(bit_nb) <= 125):
+        if not (1 <= int(bit_nb) <= 2000):
             self.__debug_msg("read_discrete_inputs() : bit_nb out of range")
             return None
         if (int(bit_addr) + int(bit_nb)) > 65536:
@@ -302,15 +334,24 @@ class ModbusClient:
             self.__debug_msg("read_discrete_inputs(): rx frame under min size")
             self.close()
             return None
-        # register extract
-        rx_byte_count = struct.unpack("B", f_body[0:1])
-        # frame with regs value
-        f_bits = f_body[1:]
-        bits = []
-        for f_byte in f_bits:
-            for pos in range(8):
-                bits.append(bool(ord(f_byte)>>pos&0x01))
-        return bits[:int(bit_nb)]
+        # extract field "byte count"
+        rx_byte_count = struct.unpack("B", f_body[0:1])[0]
+        # frame with bits value -> bits[] list
+        f_bits = bytearray(f_body[1:])
+        # check rx_byte_count: match nb of bits request and check buffer size
+        if not ((rx_byte_count == int((bit_nb+7)/8)) or
+                (rx_byte_count == len(f_bits))):
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("read_discrete_inputs(): rx byte count mismatch")
+            self.close()
+            return None
+        # allocate a bit_nb size list
+        bits = [None] * bit_nb
+        # fill bits list with bit items
+        for i, item in enumerate(bits):
+            bits[i] = bool(f_bits[int(i/8)]>>(i%8)&0x01)
+        # return bits list
+        return bits
 
     def read_holding_registers(self, reg_addr, reg_nb=1):
         """Modbus function READ_HOLDING_REGISTERS (0x03)
@@ -352,14 +393,24 @@ class ModbusClient:
                              "rx frame under min size")
             self.close()
             return None
-        # register extract
-        rx_reg_count = struct.unpack("B", f_body[0:1])
+        # extract field "byte count"
+        rx_byte_count = struct.unpack("B", f_body[0:1])[0]
         # frame with regs value
         f_regs = f_body[1:]
-        # split f_regs in 2 bytes blocs
-        registers = [f_regs[i:i+2] for i in range(0, len(f_regs), 2)]
-        registers = [struct.unpack(">H", i)[0] for i in registers]
-        return registers[:int(reg_nb)]
+        # check rx_byte_count: match nb of bits request and check buffer size
+        if not ((rx_byte_count == 2*reg_nb) or
+                (rx_byte_count == len(f_regs))):
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("read_holding_registers(): rx byte count mismatch")
+            self.close()
+            return None
+        # allocate a reg_nb size list
+        registers = [None] * reg_nb
+        # fill registers list with register items
+        for i, item in enumerate(registers):
+            registers[i] = struct.unpack(">H", f_regs[i*2:i*2+2])[0]
+        # return registers list
+        return registers
 
     def read_input_registers(self, reg_addr, reg_nb=1):
         """Modbus function READ_INPUT_REGISTERS (0x04)
@@ -400,14 +451,24 @@ class ModbusClient:
             self.__debug_msg("read_input_registers(): rx frame under min size")
             self.close()
             return None
-        # register extract
-        rx_reg_count = struct.unpack("B", f_body[0:1])
+        # extract field "byte count"
+        rx_byte_count = struct.unpack("B", f_body[0:1])[0]
         # frame with regs value
         f_regs = f_body[1:]
-        # split f_regs in 2 bytes blocs
-        registers = [f_regs[i:i+2] for i in range(0, len(f_regs), 2)]
-        registers = [struct.unpack(">H", i)[0] for i in registers]
-        return registers[:int(reg_nb)]
+        # check rx_byte_count: match nb of bits request and check buffer size
+        if not ((rx_byte_count == 2*reg_nb) or
+                (rx_byte_count == len(f_regs))):
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("read_input_registers(): rx byte count mismatch")
+            self.close()
+            return None
+        # allocate a reg_nb size list
+        registers = [None] * reg_nb
+        # fill registers list with register items
+        for i, item in enumerate(registers):
+            registers[i] = struct.unpack(">H", f_regs[i*2:i*2+2])[0]
+        # return registers list
+        return registers
 
     def write_single_coil(self, bit_addr, bit_value):
         """Modbus function WRITE_SINGLE_COIL (0x05)
@@ -581,6 +642,7 @@ class ModbusClient:
         """
         # check link, open if need
         if self.__sock is None:
+            self.__debug_msg("call _send on close socket")
             return None
         # send data
         data_l = len(data)
